import sys, os.path, shutil, getopt, glob, stat, tempfile, fnmatch, re, py_compile




_usage = """
PaK -- Python embedded packager/installer.
(c) 2003 Oleg A. Khlybov <fougas@mail.ru>

usage :
	%s option {option ...} {--} command

options :
	-h | --help       -- print usage information
	-v | --verbose    -- raise verbosity level (default is silent operation)
	--root=<new_root> -- change destination root directory to new_root (affects install/uninstall)

commands :
	sdist | build -- build distribution tarball
	install       -- install distribution
	uninstall     -- try to deinstall distribution
""" % sys.argv[0]




class Mode :
	regular		= 0644
	executable	= 0755




def _Tree (rootdir, curdir = '') :
	# NOTE : _Tree() requires that rootdir is an absolute path!
	# NOTE : _Tree() alters current working directory, so cwd must be
	# restored after it gets finished!
	dirs = []
	dir = os.path.join (rootdir, curdir)
	os.chdir (dir)
	files = glob.glob ('*') + glob.glob ('.*')
	for f in files :
		full = os.path.join (rootdir, curdir, f)
		if stat.S_ISDIR (os.stat (full)[stat.ST_MODE]) :
			relative = os.path.join (curdir, f)
			dirs.append (relative)
			dirs = dirs + _Tree (rootdir, relative)
	return dirs




class Target :
	def __init__ (self, pak) :
		self.pak = pak
		self.Setup ()
	def Setup (self) :
		self.installable = 1
	def Build (self) :
		pass
	def Install (self) :
		pass
	def Uninstall (self) :
		pass




class File (Target) :
	def Setup (self) :
		Target.Setup (self)
		self.installable = 0
	def Build (self) :
		Target.Build (self)
		for f in self.files :
			self.pak.CopyFile (f, os.path.join (self.pak.builddir, os.path.basename (f)))




class Script (Target) :
	def Setup (self) :
		Target.Setup (self)
		self.python = 1
	def Build (self) :
		Target.Build (self)
		for s in self.scripts :
			dst = os.path.join (self.pak.builddir, os.path.basename (s))
			self.pak.CopyFile (s, dst, Mode.executable)
	def Install (self) :
		if not self.installable :
			return
		Target.Install (self)
		bindir = self.pak.FixRoot (self.pak.bindir)
		self.pak.MakeDir (bindir)
		for s in self.scripts :
			src = os.path.basename (s)
			dst = os.path.join (bindir, os.path.basename (s))
			self.pak.CopyFile (src, dst, Mode.executable)
			self.pak.FixInterp (dst)
	def Uninstall (self) :
		if not self.installable :
			return
		Target.Uninstall (self)
		bindir = self.pak.FixRoot (self.pak.bindir)
		for s in self.scripts :
			dst = os.path.join (bindir, os.path.basename (s))
			self.pak.RemoveFile (dst)




class Module (Target) :
	def Setup (self) :
		Target.Setup (self)
	def Build (self) :
		Target.Build (self)
		for m in self.modules :
			if not fnmatch.fnmatch (m, '*.py') :
				self.pak.Warn ('module %s has no valid extension; .py is appended' % m)
				m = m + '.py'
			self.pak.CopyFile (m, os.path.join (self.pak.builddir, os.path.basename (m)))
	def Install (self) :
		if not self.installable :
			return
		Target.Install (self)
		libdir = self.pak.FixRoot (self.pak.libdir)
		self.pak.MakeDir (libdir)
		for m in self.modules :
			if not fnmatch.fnmatch (m, '*.py') :
				m = m + '.py'
			src = os.path.basename (m)
			dst = os.path.join (libdir, os.path.basename (m))
			vdst = os.path.join (self.pak.libdir, os.path.basename (m))
			self.pak.CopyFile (src, dst)
			py_compile.compile (file = dst, dfile = vdst)
			self.pak.Log ('compiled %s' % dst)
	def Uninstall (self) :
		if not self.installable :
			return
		Target.Uninstall (self)
		libdir = self.pak.FixRoot (self.pak.libdir)
		for m in self.modules :
			g = os.path.splitext (os.path.join (libdir, os.path.basename (m)))[0] + '.py*'
			for f in glob.glob (g) :
				self.pak.RemoveFile (f)




class Package (Target) :
	def Setup (self) :
		Target.Setup (self)
	def Build (self) :
		Target.Build (self)
		for p in self.packages :
			name = os.path.basename (p)
			root = os.path.dirname (os.path.abspath (p))
			cwd = os.getcwd ()
			dirs = _Tree (os.path.join (root, name))
			os.chdir (cwd)
			for d in [''] + dirs :
				self.pak.MakeDir (os.path.join (self.pak.builddir, name, d))
				for src in glob.glob (os.path.join (root, name, d, '*.py')) :
					f = os.path.basename (src)
					dst = os.path.join (self.pak.builddir, name, d, f)
					self.pak.CopyFile (src, dst)
	def Install (self) :
		if not self.installable :
			return
		Target.Install (self)
		for p in self.packages :
			name = os.path.basename (p)
			root = os.path.join (os.getcwd (), name)
			cwd = os.getcwd ()
			dirs = _Tree (root)
			os.chdir (cwd)
			libdir = self.pak.FixRoot (self.pak.libdir)
			for d in [''] + dirs :
				self.pak.MakeDir (os.path.join (libdir, name, d))
				for src in glob.glob (os.path.join (name, d, '*.py')) :
					f = os.path.basename (src)
					dst = os.path.join (libdir, name, d, f)
					self.pak.CopyFile (src, dst)
					vdst = os.path.join (self.pak.libdir, name, d, f)
					py_compile.compile (file = dst, dfile = vdst)
					self.pak.Log ('compiled %s' % dst)
	def Uninstall (self) :
		if not self.installable :
			return
		Target.Uninstall (self)
		for p in self.packages :
			name = os.path.basename (p)
			libdir = self.pak.FixRoot (self.pak.libdir)
			self.pak.RemoveDir (os.path.join (libdir, name))




class Utils :
	shbpython = re.compile ('^\#\!.*python')
	shbany    = re.compile ('^\#\!')
	def Log (self, msg) :
		if self.verbosity > 0 :
			sys.stdout.write ('--- PaK : %s\n' % msg)
	def Warn (self, msg) :
		sys.stderr.write ('=== PaK : %s\n' % msg)
	def Err (self, msg) :
		sys.stderr.write ('*** PaK : %s\n' % msg)
		sys.exit (1)
	def CopyFile (self, src, dst, mode = Mode.regular) :
		shutil.copyfile (src, dst)
		self.Log ('%s --> %s' % (src, dst))
		os.chmod (dst, mode)
	def MakeDir (self, dir) :
		try :
			os.makedirs (dir)
			self.Log ('created %s' % dir)
		except OSError, (errno, strerror) :
			if not (errno == 17) :
				raise
	def RemoveDir (self, dir) :
		try :
			shutil.rmtree (dir)
			self.Log ('removed %s' % dir)
		except OSError, (errno, strerror) :
			if not (errno == 2) :
				raise
	def RemoveFile (self, file) :
		try :
			os.unlink (file)
			self.Log ('removed %s' % file)
		except OSError, (errno, strerror) :
			if not (errno == 2) :
				raise
	def FixInterp (self, script) :
		id = open (script, 'rt')
		try :
			line = id.readlines ()
		finally :
			id.close ()
		if self.force :
			if Utils.shbany.match (line[0]) :
				pass
			else :
				line.insert (0, '\n') # Simply insert empty string which will turn into #!... later
		else :
			if not Utils.shbpython.match (line[0]) :
				return
		line[0] = '#!%s\n' % self.python
		id = open (script, 'wt')
		try :
			id.writelines (line)
		finally :
			id.close ()
		self.Log ('fixed #! in %s' % script)
	def FixRoot (self, path) :
		if self.rootdir :
			return os.path.normpath (self.rootdir + '/' + path)
		else :
			return path
	def PaKSource (self) :
		i = __file__.rfind ('.')
		if i < 0 :
			self.Err ('file %s has no extension' % __file__)
		head = __file__[:i]
		tail = __file__[i + 1:]
		if tail in ('py', 'pyc', 'pyo') :
			tail = 'py'
		else :
			self.Err ('%s is not valid Python module extension' % tail)
		src = '%s.%s' % (head, tail)
		if not os.path.isfile (src) :
			self.Err ('PaK source module not found')
		return src




class PaK (Utils):
	def __init__ (self) :
		self.tarballdir = '.'
		self.rootdir = None
		self.verbosity = 0
		self.force = 1
		self.Setup ()
		self.pkgname = '%s-%s' % (self.package, self.version)
		self.pkgfile = '%s.tar.gz' % self.pkgname
		self.targets = []
		for z in self.__dict__.values () :
			try :
				if issubclass (z, Target) :
					self.targets.append (z (self))
			except TypeError :
				pass
	def __call__ (self) :
		build = 0
		install = 1
		uninstall = 2
		try :
			opts, args = getopt.getopt (sys.argv[1:], 'hv', ['help', 'verbose', 'root='])
		except getopt.GetoptError :
			print _usage
			sys.exit (2)
		largs = len (args)
		if largs > 1 :
			self.Warn ('too many commands issued; the last one wins')
		if largs > 0 :
			cmd = args[largs - 1]
			if   cmd == 'sdist' or cmd == 'build' :
				action = build
			elif cmd == 'install' :
				action = install
			elif cmd == 'uninstall' :
				action = uninstall
			else :
				self.Err ('unknown command: %s' % cmd)
		else :
			action = -1
		for o, a in opts :
			if o in ('-h', '--help') :
				print _usage
				sys.exit ()
			if o in ('-v', '--verbose') :
				self.verbosity = self.verbosity + 1
			if o in ('--root',) :
				self.rootdir = a
				if action == build :
					self.Warn ('inappropriate option %s for the command; option is ignored' % o)
		if action < 0 :
			print _usage
			sys.exit ()
		if   action == build :
			self.Build ()
		elif action == install :
			self.Install ()
		elif action == uninstall :
			self.Uninstall ()
	def Setup (self) :
		self.prefix = os.path.normpath (sys.prefix)
		self.python = sys.executable
		for _t in sys.path :
			if _t.find ('site-packages') >= 0 and os.path.isdir (_t) :
				libdir = _t
				break
		try :
			libdir
			self.libdir = libdir
		except :
			self.Err ('failed to determine where to install Python modules')
		_t = os.path.join (self.prefix, 'bin')
		if not os.path.isdir (_t) :
			self.Err ('non-existent directory %s' % _t)
		else :
			self.bindir = _t
		self._pak_ = _PaK
		self._setup_ = _Setup
	def Build (self) :
		self.Log ('building...')
		tempdir = tempfile.mktemp ('.PaK')
		self.builddir = os.path.join (tempdir, self.pkgname)
		self.MakeDir (self.builddir)
		try :
			for t in self.targets :
				t.Build ()
			tarball = os.path.abspath (os.path.join (self.tarballdir, self.pkgfile))
			cmd = 'cd %s && tar czf %s *' % (tempdir, tarball)
			os.system (cmd)
			self.Log ('built %s' % tarball)
		finally :
			self.RemoveDir (tempdir)
		del self.builddir
		self.Log ('building successful')
	def Install (self) :
		self.Log ('installing...')
		for t in self.targets :
			t.Install ()
		self.Log ('installing successful')
	def Uninstall (self) :
		self.Log ('deinstalling...')
		for t in self.targets :
			t.Uninstall ()
		self.Log ('denstalling successful')




class _PaK (Module) :
	def Setup (self) :
		Module.Setup (self)
		self.installable = 0
		self.modules = [self.pak.PaKSource ()]




class _Setup (Script) :
	def Setup (self) :
		Script.Setup (self)
		self.installable = 0
		self.scripts = [sys.argv[0]]
